/*
  Project:  4x3 matrix membrane keypad,LEDs, LCD1602 I2C module,active piezo buzzer, servo, 5v relay module / security code
  Function: This sketch takes input from a keypad for secret code.  If the code is
           entered correctly it turns on a green LED to simulate access granted and
           prints access granted in Serial Monitor and on LCD screen. Otherwise the red LED is lit and
           access denied is printed in Serial Monitor and on LCD screen. Equipment connected to DC 5v relay will
           turn on if security code is correct.Servo motor will turn 90 degrees too.
           */
// Keypad Library for Arduino
// Authors:  Mark Stanley, Alexander Brevig
// http://playground.arduino.cc/Main/KeypadTutorial
//******************************************************************************************
#include <Password.h>
#include <Keypad.h>
#include "pitches.h"//include library code
#include <LiquidCrystal_I2C.h>//include library code
#include <Servo.h>//include library code
//******************************************************************************************
// Melodies definition: access and rejection - you can change melodies. see in file pitches.h
int keypadRead;  // keypad read 1 = good, 0 = bad for playTune function
int access_melody[] = { NOTE_C4, NOTE_G3, NOTE_G3, NOTE_A3, NOTE_G3,0, NOTE_B3, NOTE_C4};
int access_noteDurations[] = {4,8,8,4,4,4,4,4};
int noaccess_melody[] = {NOTE_G2, NOTE_G2};
int noaccess_noteDurations[] = {8,8};
const int buzzerPin = 12;           //active piezo buzzer attached to digital pin 12
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);//Initialise the LCD 1602 I2C module
const int relayPin = 14; //5V relay module attached to analog pin 1  
const int servoPin = 13;      //servo attached to digital pin 13
// note that in Arduino 0016 and earlier, the Servo library supports only servos on only two pins: 9 and 10
Servo Lock; //servo motor definition
const int greenledPin = 10;   //green LED attached to digital pin 10
const int redledPin = 11;     //red LED attached to digital pin 11
Password password = Password( "2468" ); // Set password as 2468
const byte ROWS = 4; //four rows
const byte COLS = 3; //three columns
char hexaKeys[ROWS][COLS] = {//define the symbols on the buttons of the keypads
{'1','2','3'},
{'4','5','6'},
{'7','8','9'},
{'*','0','#'}
};
byte rowPins[ROWS] = {9, 8, 7, 6}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {5, 4, 3};    //connect to the column pinouts of the keypad
Keypad keypad = Keypad( makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS);//initialize an instance of class NewKeypad
int i = 0;
//******************************************************************************************
void setup() {
  pinMode(buzzerPin,OUTPUT);            //sets buzzerPin as OUTPUT
  pinMode(greenledPin, OUTPUT);         //sets greenledPin as OUTPUT
  pinMode(redledPin, OUTPUT);           //sets redledPin as OUTPUT
  Serial.begin(9600);                   //initialise the serial communication at 9600 bps
  pinMode(relayPin, OUTPUT);             //sets relayPin as OUTPUT
  Lock.attach(servoPin);                 // Servo motor attached to the servoPin
  pinMode(servoPin, OUTPUT);             //sets servoPin as OUTPUT
  Lock.write(0);//sets initial Servo position - LOCKED MODE
  lcd.begin (16, 2);//Define the LCD as 16 column by 2 rows
  lcd.setBacklightPin(3, POSITIVE);//Switch on the backlight
  lcd.setBacklight(HIGH);
  lcd.print("***Electronic***");//Print at cursor Location
  lcd.setCursor(0, 1); //goto first column (column 0) and second line (line 1)
  lcd.print("***Keypad Lock**");//Print at cursor Location
  Serial.println("********************");
  Serial.println("*****Electronic*****");
  Serial.println("*****Keypad Lock****");
  Serial.println("********************");
  delay(2000);
  lcd.clear();
  lcd.print("Enter code + * :");
  lcd.setCursor(0, 1); //goto first column (column 0) and second line (line 1)
  lcd.blink();
  Serial.println("Enter code, press * :");
  keypad.addEventListener(keypadEvent); // Add an event listener to
  // detect keypresses
}
void loop() {
  
  keypad.getKey();

  
}
void keypadEvent(KeypadEvent eKey) {

  switch (keypad.getState()) {
    case PRESSED:

      lcd.setCursor(i, 1);
      lcd.print(eKey);
      i++;
      Serial.print("Pressed key: ");
      Serial.println(eKey);
      switch (eKey) {
        case '*':
          checkPassword();
          break;
        case '#':
          digitalWrite(relayPin, HIGH);//deactivates the equipment connected to DC 5V relay module as NO terminal will connect to Common terminal
          Lock.write(0); //locks the door, you need to adjust this to positioning the servo according your door locker
     
          lcd.clear();
          lcd.print("Enter code + * :");
          lcd.setCursor(0, 1); //goto first column (column 0) and second line (line 1)
          lcd.blink();
          Serial.println("Enter code, press * :");
          password.reset();
          i = 0;
          break;
        default:
          password.append(eKey);
      }
  }
}
//*****************************************************************
void checkPassword() {
  if (password.evaluate() ) {
    digitalWrite(relayPin, LOW);//activates the equipment connected to DC 5V relay module as NO terminal will connect to Common terminal
    Lock.write(90); //releases the door, you need to adjust this to positioning the servo according your door locker
      
      keypadRead = 1;
    playTune(keypadRead);
    digitalWrite(greenledPin, HIGH); // Turn on green LED
    lcd.clear();
    lcd.print("***Electronic***");//Print at cursor Location
    lcd.setCursor(0, 1); //goto first column (column 0) and second line (line 1)
    lcd.print("*Access granted*");
    Serial.println("");// If the password is correct...
    Serial.println("********************");
    Serial.println("** ACCESS GRANTED **");
    Serial.println("**   WELCOME!!    **");
    Serial.println("********************");
    delay(2000);//sets delay for 2 seconds
    lcd.clear();
    lcd.print("Press # to reset");//Print at cursor Location
    Serial.println("Press # to reset");
    digitalWrite(greenledPin, LOW); // Turn off green LED

  } else {
    keypadRead = 0;
    playTune(keypadRead);
    digitalWrite(redledPin, HIGH); // Turn on red LED
    lcd.clear();
    lcd.print("***Electronic***");//Print at cursor Location
    lcd.setCursor(0, 1); //goto second column (column 2) and second line (line 1)
    lcd.print("**Access denied*");
    Serial.println("");// If the password is incorrect...
    Serial.println("********************");
    Serial.println("** ACCESS DENIED! **");
    Serial.println("**  INVALID CODE  **");
    Serial.println("********************");
    delay(2000);//sets delay for 2 seconds
    lcd.clear();
    lcd.print("Press # to reset");//Print at cursor Location
    Serial.println("Press # to reset");
    digitalWrite(redledPin, LOW); // Turn off red LED
  }
}
//**************************************************************
//Function to play the melody during Access and No access
void playTune(int Scan) {
  if (Scan == 1) // Correct secret code entered - Access granted
  {
    for (int i = 0; i < 8; i++)    //loop through the notes
    { // Good secret code read
      int access_noteDuration = 1000 / access_noteDurations[i];
      tone(buzzerPin, access_melody[i], access_noteDuration);
      int access_pauseBetweenNotes = access_noteDuration * 1.30;
      delay(access_pauseBetweenNotes);
      noTone(buzzerPin);
    }
  }
  else // Wrong secret code entered - No Access
    for (int i = 0; i < 2; i++)    //loop through the notes
    {
      int noaccess_noteDuration = 1000 / noaccess_noteDurations[i];
      tone(buzzerPin, noaccess_melody[i], noaccess_noteDuration);
      int noaccess_pauseBetweenNotes = noaccess_noteDuration * 1.30;
      delay(noaccess_pauseBetweenNotes);
      noTone(buzzerPin);
    }
}
